"""Top-level package for Smart Farm Hub."""

__author__ = """Joseph Malemela"""
__email__ = 'itukzz96@gmail.com'
__version__ = '0.1.7'


from LowCostSmartFarmHub.sensor import Sensor
from LowCostSmartFarmHub.actuator import Actuator
from LowCostSmartFarmHub.gateway import Gateway
from LowCostSmartFarmHub.nodeDevice import NodeDevice




from    digi.xbee.devices   import XBeeDevice
from    digi.xbee.io    import  IOLine,IOMode
import time

class Sensor:
    '''This class provides functionality for the Sensor'''
    sensor_name:str
    sensor_id:int
    sensor_type:str
    sensor_values:[]
    description:str
    unit_of_measure:str
    
    def __init__(self,sensorName,sensorID,sensorType,description,unit_of_measure,sensorValues=[]):
        self.sensor_name=sensorName      #Every Sensor on the network has a name
        self.sensor_id=sensorID          #Every Sensor on the network has a unique ID
        self.sensor_type=sensorType      #I2C,ADC,DIO
        self.sensor_values=sensorValues  #[recent sensor values]
        self.description=description    #More information about sensor ,humidity,Temperature
        self.unit_of_measure=unit_of_measure    #The unit of measure (percentage,degrees,grams of water per unit of air)
        
    def read_analog_xbee_sensor(self,XbeeDevice:XBeeDevice,xbee_pin_index,analog_conversion):
        """
        This provides functionality for getting the sensor value on a Xbee Node

        Args:
            analog_pin_index (Integer) : The analog pin that sensor is connected to on XBee module 0=>DIO0,1->DIO1
            XbeeDevice (XBeeDevice)    : The node device where the sensor is connected
        
        Returns:
            Sensor Value
        """
        sensor_value=0

        #Configure the  pin to analog (Pin must support Analog signal Pin 0-Pin3)
        XbeeDevice.set_io_configuration(IOLine.get(xbee_pin_index),IOMode.ADC)
        sensor_value=XbeeDevice.get_adc_value(IOLine.DIO0_AD0)

        #Convert 10 Bit ADC value to relevant value
        sensor_value=round(float(sensor_value/1023.0)*analog_conversion,2) 

        #raise Exception('The selected pin does not support Analog');
        
        self.sensor_values.append(sensor_value)
        return str(sensor_value)

    
    def read_digital_xbee_sensor(self,xbee_device:XBeeDevice,io_digital_pin):
        """
        This function provides functionality for interfacing with the DHT11 Humidity and Temperature sensor 
        connected to an XBee 3 module

        Args:
            xbee_device (XBee Device): The Xbee module object that represents the XBee module 3 in the network
            io_digital_pin (Integer) : The digital IO pin that the sensor is connected to
        """
    def get_sensor_value(self):
        """
        Gets the most recent sensor value 

        Returns:
            Sensor value
        """
        return self.sensor_values[len(self.sensor_values)-1]



#RPI:3B+ Gateway

'''Pin connection for sensors,actuators and Coordinator Node
   Pin 1        : 3.3V <--->Sensor 1,2,3 (VCC)
   Pin 2        : 5V   <--->Low Voltage Actuator
   Pin 11,13,15 : GPIO <--->Digital Sensor Inputs
   Pin 9,30     : GND  <--->Sensor and Actuator GND
   USB          : USB  <--->UART Xbee 
'''

from LowCostSmartFarmHub.sensor import Sensor
from LowCostSmartFarmHub.actuator import Actuator
from LowCostSmartFarmHub.nodeDevice import NodeDevice
import  time
from    digi.xbee.devices   import XBeeDevice
import  serial
import  json
from paho.mqtt import client as mqtt_client

class Gateway:
    '''This class provides functionality for the Gateway Device'''
    deviceName:str              #The unique name for the gateway device
    sensors:[]                  #All the sensors connected directly to sensor
    actuators:[]                #All the actuators connected directly to gateway
    location:str                #Locaion of the gateway
    nodeDevices:[]              #All the Node Devices(Xbee) in the Network
    panID:str                   #The Unique Zigbee Network Identifier
    localXBee:XBeeDevice        #The Local Zigbee device used as Coordinator through serial Port


    def __init__(self,deviceName,location,sensors=None,actuators=None,nodeDevices=None,panID=None):
        self.deviceName=deviceName
        self.sensors=sensors
        self.actuators=actuators
        self.location=location
        self.nodeDevices=nodeDevices
        self.panID=panID



    def add_sensor(self,sensor:Sensor):
        """
        Adds a sensor directly to the gateway Device

            Args:
                sensor(Sensor): The sensor object with all the sensor attributes
            
            Returns: 
                A list of all the sensors connected directly to the gateway
        """
        self.sensors.append(sensor)

    def add_actuator(self,actuator:Actuator):
        """
        Adds an Actuator directly to the gateway Device

            Args:
                actuator(Actuator): The actuator object with all the actuator attributes
            
            Returns: 
                A list of all the actuators connected directly to the gateway
        """
        self.actuators.append(actuator)

    def add_zigbee_device(self,nodeName,nodeType,xbeeDevice:XBeeDevice):
        """
        Adds the  Zigbee Device to  the Gateway Node Devices
	        Args:
             	node_name (String): name of zigbee device
             	node_type (String): the type of node-device (end-device,router-device,coordinator-device)
             	xbee_device (XBeeDevice): the xbee device of object if device is a Digi XBee module

        	Returns:
             	List of Node Devices connected to Gateway wirelessly and through the serial Port
        	Raises:
             	XBeeException:if the provided XBee device does not responds to commands
        	"""

        print("Adding Zigbee Device to Network")
        print(xbeeDevice.read_device_info())
        #xnet=self.localXBee.get_network()
        #xnet.add_remote(xbeeDevice)
        
        #nodeDevice.macAddress=
        #nodeDevice.XbeeObject=XBeeDevice
        #nodeDevice=NodeDevice(nodeName,nodeType)
    
        self.nodeDevices.append(nodeDevice("","",""))

    def connect_stream_uart(self,comPort,baud_rate,discover_devices):
        """
		Function for opening serial communication Port between RPI and  XBee Device
		Args:
			com_port (String): The serial port where the Coordinator Device is connected to on the Gateway
		Returns:
			None
    	"""
        localXBee=XBeeDevice(comPort,baud_rate)  #With Baudrate:9600
        localXBee.open()
        self.panID=localXBee.get_pan_id()   #Set the PanID
        self.localXBee=localXBee
	

    def discover_zigbee_devices(self):
        """
        Discovers new zigbee devices on the zigbee network 

		Args:
            self
		Returns:
		    A list of the Discovered XBee Devices
        """
        #Get Xbee network object from the Xbee Device
        xnet=self.localXBee.get_network()
        #Clear List of of Devices for Clean Discovery
        xnet.clear()
        #Start the discovery process and wait for it to be over
        xnet.start_discovery_process()
        while xnet.is_discovery_running():
            time.sleep(1.5)
        #Get the List of Devices added to the Network and Add to Node Devices        
        devices = xnet.get_devices()
        xnet.add_remotes(devices)
        return  devices

    def publish_sensor_info(self,client,sensor:Sensor):
        """
        Publishes the provided sensor infromation to the broker specified

        Args:
            client(mqtt_client):The MQTT client object
            sensor(Sensor): The sensor object with all the attributes of the sensor
            port(int): The port where the MQTT broker is running
            broker_address: The Address of the MQTT broker e.g (localhost or 192.168.10.1 or www.mqtt-broker.io)

        """
        payload_dict={"sensor_name":sensor.sensor_name,"sensor_id":sensor.sensor_id,"sensor_connection":"ADC","data":{"value":sensor.get_sensor_value(),"units":sensor.unit_of_measure}}
        payload=json.dumps(payload_dict)

        topic = 'data/myfarm/dorm-room/'+sensor.sensor_name+"/"

        result = client.publish(topic,payload,2)

        # result: [0, 1]
        status = result[0]
        if status == 0:
            print(f"Send `{payload}` to topic `{topic}`")
        else:
            print(f"Failed to send message to topic {topic}")
        
    def connect_mqtt(self,client_id,broker,port):
        """
        This function connects to the MQTT broker

        Args:
            client_id(int) : unique identifier of MQTT client
            broker (string): broker address
            port (int) : The port where the mqtt broker is running
        
        Returns:
            mqtt client 
        """
        def on_connect(client, userdata, flags, rc):
            if rc == 0:
                client.subscribe("cmd/#")   #Subscribe to Commands from MQTT broker
                print("Connected to MQTT Broker!")
            else:
                print("Failed to connect, return code %d\n", rc)
        
        def on_message(client, userdata, msg):
             print(msg.topic+" "+str(msg.payload))

        # Set Connecting Client ID
        client = mqtt_client.Client(client_id)
        client.on_connect = on_connect
        client.on_message = on_message
        client.connect(broker, port)
        return client

       #def update_fiirmware(XBeeDevice:xbee_device):
        #Update Local Xbee Device 
        #Update Remote Xbee Device



import paho.mqtt.client as mqtt
from    gateway import  Gateway
import time
from    nodeDevice import NodeDevice
from    sensor  import Sensor
import  json
import  random
#The callback for when the client receives a CONNACK response from the server.

gateway=Gateway("RPI","farm1",[],[],[],"")

def main():

    #Initialize Gateway object from gateway
    print("Connecting to Local XBee through UART")
    gateway.connect_stream_uart("/dev/serial0",9600,True)
    
    devices=gateway.discover_zigbee_devices()
    print("Remote Xbee Devices: ",devices)

    #Initialize all sensors on the network 
    sensor1=Sensor("Soil Moisture Sensor","XCVE","Soil Moisture","Measures soil moisture in percentage","%")
    node_device_1=NodeDevice("Remote Xbee Module","end-device","GBSJDMMD",sensor1)
    node_device_1.XbeeObject=devices[0]

    sensor2=Sensor("Temperature Sensor","BFNND","Temperature","Measures Temperature in degrees celcius","degrees")
    node_device_2=NodeDevice("Local Xbee Module","end-device","GBSJDMMD",sensor2)
    node_device_2.XbeeObject=gateway.localXBee

    sensor3=Sensor("Humidity Sensor","BFNND","Humidity","Measures Humidity in percentage","%")


    print("Publishing to Broker every minute")

    client_id = f'python-mqtt-{random.randint(0, 1000)}'
    mqtt_client=gateway.connect_mqtt(client_id,'localhost',1883)
    mqtt_client.loop_start() 
    while True:
      #Read Sensor Values
      sensor_value=sensor1.read_analog_xbee_sensor(node_device_1.XbeeObject,0,100)
      gateway.publish_sensor_info(mqtt_client,sensor1)

      sensor_value2=sensor2.read_analog_xbee_sensor(node_device_2.XbeeObject,0,50)
      gateway.publish_sensor_info(mqtt_client,sensor2)
      
      sensor_value3=sensor3.read_analog_xbee_sensor(node_device_3.XbeeObject,0,100)

      gateway.publish_sensor_info(mqtt_client,sensor3)

      time.sleep(60)    #Sleep for a minute

if __name__ == '__main__':
    print('Testing MQTT Client Functions')
    main()



import time
from rpi_ws281x import Color, PixelStrip, ws

class Actuator:
    actuatorName:str
    actuatorID:int
    actuatorType:str
    actuatorValues:[]
    description:str
    def __init__(self,actuatorName,actuatorID,actuatorType,description,actuatorValues=None):
        self.actuatorName=actuatorName      #Every Actuator on the network has a name
        self.actuatorID=actuatorID          #Every Actuator on the network has a unique ID
        self.actuatorType=actuatorType      #DIO,DAC
        self.actuatorValues=actuatorValues  #[recent actuator commands]
        self.description=description        #More information about actuator ,humidity,Temperature

    def control_ws28x1_light(self,led_pin,state):
        """
        Function to  configure and control the ws28x1 RGB LED Strip
       
        Args:
	    led_pin:GPIO pin connected to the Pixels from RPI  (must Support PWM)
        Returns:
            current state of the light
        """
        # LED strip configuration:
        LED_COUNT = 32        # Number of LED pixels.
        LED_PIN = led_pin     # GPIO pin connected to the pixels (must support PWM!).
        LED_FREQ_HZ = 800000  # LED signal frequency in hertz (usually 800khz)
        LED_DMA = 10           # DMA channel to use for generating signal (try 10)
        LED_BRIGHTNESS = 10   # Set to 0 for darkest and 255 for brightest
        LED_INVERT = False    # True to invert the signal (when using NPN transistor level shift)
        LED_CHANNEL = 0
        # LED_STRIP = ws.SK6812_STRIP_RGBW
        LED_STRIP = ws.SK6812W_STRIP
  
       # Create PixelStrip object with appropriate configuration.
        strip = PixelStrip(LED_COUNT, LED_PIN, LED_FREQ_HZ, LED_DMA, LED_INVERT, LED_BRIGHTNESS, LED_CHANNEL, LED_STRIP)
        # Intialize the library (must be called once before other functions).
        strip.begin()
        if(state=="on"):
          strip.setPixelColor(0,Color(4,255,255))

        elif (state=="off"):
          strip.setPixelColor(0,Color(0,0,0))
        strip.show()



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = ['Click>=7.0', ]

setup_requirements = [ ]

test_requirements = [ ]

setup(
    author="Joseph Malemela",
    author_email='itukzz96@gmail.com',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="A gateway API for controlling and monitoring a wireless sensor network Digi-Xbee Modules",
    entry_points={
        'console_scripts': [
            'LowCostSmartFarmHub=LowCostSmartFarmHub.cli:main',
        ],
    },
    install_requires=[
        'digi-xbee',
        'click',
        'rpi_ws281x',
        'paho-mqtt',
    ],
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='LowCostSmartFarmHub',
    name='LowCostSmartFarmHub',
    packages=find_packages(include=['LowCostSmartFarmHub','LowCostSmartFarmHub/','LowCostSmartFarmHub.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/itumeleng96/LowCostSmartFarmHub',
    download_url= 'https://github.com/itumeleng96/LowCostSmartFarmHub/archive/0.1.9.tar.gz',
    version='v0.1.9',
)





#Pin Specification for Xbee3 Node Device
#VCC 
#GND
#Pin 2      : DIO13     <---->UART Data Out
#Pin 3      : DIO14     <---->UART Data In
#Pin 17-20  : DIO3-DIO0 <---->Sensor Analog/Digital Input

#This Class defines the Node Devices on the Network

from    digi.xbee.devices   import XBeeDevice
from    LowCostSmartFarmHub.sensor  import Sensor

class NodeDevice:
    '''This class represents every node device on the network'''
    nodeName:str
    nodeType:str
    location:str
    sensors:[]
    actuators:[]
    macAddress:str
    batteryLevel:str
    XbeeObject:XBeeDevice

    # Initialize Node Object	
    def __init__(self,nodeName,nodeType,macAddress,sensors=None,actuators=None,location=None,batteryLevel=None):
        self.nodeName=nodeName              #Xbee3 node,zigbee device
        self.nodeType=nodeType              #Coordinator,End-Device,Router
        self.location=location              #Location of node device,e.g Farm1 lat:28298383 lng:937373
        self.sensors=sensors                #A list of all the sensors connected on the Node
        self.actuators=actuators            #A list of all the actuators connected on the Node
        self.batteryLevel=batteryLevel      # Device battery level in decimal
        self.macAddress=macAddress          # Every Device or Node on network has MAC Address

    def add_sensor(self,sensor):
        """
        To add sensors to the node

        Args:
            sensor(Sensor): The sensor Object containing all attributes of the sensor
        
        Returns:
            the new list of sensors
        """
        self.sensors.append(sensor)
        #Trigger some event to read sensor Value and send to Broker

    def remove_sensor(self,sensorID):
        """
        To remove  sensors from the node

        Args:
            sensor(Sensor): The sensor Object containing all attributes of the sensor
        
        Returns:
            the new list of sensors
        
        Throws Exception:
            if the sensor is not in the list
        """
        for sensor in self.sensors:
            if sensor.sensorID==sensorID:
                self.sensors.remove(sensor)

    def add_actuator(self,actuator):
        """
        To add actuators to the node

        Args:
            actuator(Actuator): The actuator Object containing all attributes of the sensor
        
        Returns:
            the new list of actuators
        """
        self.actuators.append(actuator)

    def removeActuator(self,actuatorID):
        """
        To remove  actuators from the node

        Args:
            actuator(Actuator): The actuator Object containing all attributes of the sensor
        
        Returns:
            the new list of actuators
        """
        for actuator in self.actuators:
            if actuator.actuatorID==actuatorID:
                self.actuators.remove(actuator)

     
